/*

Disk image transfer for Serial Slave

*/


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <rs232.h>
#include <errno.h>
#ifdef __C64__
#include <c64.h>
#endif
#ifdef __C128__
#include <c128.h>
#endif
#ifdef __PLUS4__
#include <plus4.h>
#endif

#include "ssdisk.h"
#include "geometry.h"
#include "dir.h"
#include "sscmd.h"
#include "diskio.h"
#include "serial.h"
#include "ssui.h"


unsigned char cksum[MAXBLOCKS];
unsigned char direction;

unsigned char *titlestring = "\x12Serial Slave disk transfer v0.1\xa9\xa9\xa9\xa9\xa9\xa9\xa9\xa9x\x92\n";


/*

Reset timer

*/
void resettimer() {
#if defined(__C64__) || defined(__C128__)
  CIA1.tod_hour = 0x00;
  CIA1.tod_min = 0x00;
  CIA1.tod_sec = 0x00;
  CIA1.tod_10 = 0;
#endif
#ifdef __PLUS4__
  __asm__("sei");
  (*(unsigned char*)0x00a3) = 0;
  (*(unsigned char*)0x00a4) = 0;
  (*(unsigned char*)0x00a5) = 0;
  __asm__("cli");
#endif
}


/*

Get number of tenths of seconds since timer was reset

*/
unsigned int readtimer() {
#if defined(__C64__) || defined(__C128__)
  unsigned int tenths;
  tenths = CIA1.tod_hour; /* dummy read to stop TOD registers */
  tenths = CIA1.tod_min * 600;
  tenths += CIA1.tod_sec * 10;
  tenths += CIA1.tod_10;
  return(tenths);
#endif
#ifdef __PLUS4__
  unsigned int tenths;
  __asm__("sei");
  tenths = (*(unsigned char*)0x00a4) * 256;
  tenths += (*(unsigned char*)0x00a5);
  __asm__("cli");
  return(tenths/6);
#endif
}


/*

Write an image from SS to a drive

*/
unsigned char writeimage(unsigned char *name, unsigned char device, unsigned char verify, unsigned char *buffer) {
  unsigned int numblocks;
  unsigned char t;
  unsigned int timer;
  unsigned char msg[31];

  resettimer();

  if ((numblocks = blockload(name)) == 0) {
    sprintf(msg, "Couldn't load %s\n", name);
    (void) showrequester("Error", msg, REQ_OK);
    return(ERROR);
  }

  resetscreen(titlestring);
  printf("Buffered %d blocks\n", numblocks);

  for (t = 1; t <= g1541[0]; ++t) {

    if (kbhit() > 0) {
      if (cgetc() == A_BREAK) {
	printf("Aborted.");
	return(ERROR);
      }
    }

    printf("Reading %-2d: ", t);
    if (readblocks(buffer, g1541offset[t], g1541[t]) != OK) {
      printf("failed\n");
      return(ERROR);
    } else {
      printf(" OK\n");
    }

    gotoy(wherey() - 1);

    printf("Writing %-2d: ", t);
    if (writetrack(device, buffer, t, g1541[t], D1541) != OK) {
      printf("failed\n");
      return(ERROR);
    } else {
      printf(" OK\n");
    }

    if (verify) {
      gotoy(wherey() - 1);

      printf("Verify  %-2d: ", t);
      if (verifytrack(device, buffer, t, g1541[t], D1541) != OK) {
	printf("failed\n");
	return(ERROR);
      } else {
	printf(" OK\n");
      }
    }
  }


  if (sendi0(device) != OK) {
    sprintf(msg, "I0 command failed on device %d", device);
    (void) showrequester("Warning", msg, REQ_OK);
  }

  timer = readtimer();
  printf("Operation took %d.%d seconds\n", timer / 10, timer % 10);
  return(OK);
}


/*

Change directory

*/
unsigned char changedir(unsigned char *name) {
  if (direction == SSTOCBM) {
    return(changessdir(name));
  } else {
    return(ERROR);
  }
}


/*

Disk image utility

*/
int main(void) {
  unsigned char type;
  unsigned char c;
  unsigned char *buffer;
  unsigned char status;
  directory *dir;
  direntry *de;
  iecdrive *drv;
  iecdrive *d;
  unsigned char numdrives;
  unsigned char device;
  unsigned char verify = 0;
  unsigned char ssver[40];
  unsigned char msg[39];
#if defined(__C64__) || defined(__C128__)
  unsigned char interface = SER_TURBO232;
  unsigned char baudrate = RS_BAUD_115200;
#endif
#ifdef __PLUS4__
  unsigned char interface = SER_TURBO232;
  unsigned char baudrate = RS_BAUD_19200;
#endif
  unsigned char screenw, screenh;
  unsigned char *scr;

  direction = SSTOCBM;
  screensize(&screenw, &screenh);

  if ((buffer = malloc(MAXBLOCKS * 256)) == NULL) {
    printf("Couldn't allocate %d byte buffer\n", MAXBLOCKS * 256);
    return(EXIT_FAILURE);
  }

  if ((status = setupgeometry()) != OK) {
    return(EXIT_FAILURE);
  }

  resetscreen(titlestring);

#if defined(__C64__) || defined(__C128__)
  if (configserial(&interface, &baudrate) != OK) {
    return(EXIT_SUCCESS);
  }
#endif

  if (ser_init(interface, baudrate) != OK) {
    return(EXIT_FAILURE);
  }

  if (pingss(&ssver) != OK) {
    (void) showrequester("Error", "No SS server!", REQ_OK);
    ser_exit();
    return(EXIT_FAILURE);
  }

  if ((drv = scandrives()) == NULL) {
    (void) showrequester("Error", "No drives found!", REQ_OK);
    ser_exit();
    return(EXIT_FAILURE);
  }

  d = drv;
  numdrives = 1;
  while (d->next) {
    ++numdrives;
    d = d->next;
  }

  sprintf(msg, "Found %d drives.", numdrives);
  (void) showrequester(ssver, msg, REQ_OK);

  device = drv->dev;

  if ((dir = getdir(direction, buffer, device)) == NULL) {
    (void) showrequester("Error","Couldn't read directory.", REQ_OK);
    ser_exit();
    return(EXIT_FAILURE);
  }

  /*
  if ((dir = getdir(CBMTOSS, buffer, 8)) == NULL) {
    (void) showrequester("Error","Couldn't read directory.", REQ_OK);
    return(EXIT_FAILURE);
  }
  */


  sprintf(msg, "\x12dev: %2d \x92", device);
  gotoxy(screenw - 9, 2);
  myputs(msg);

  gotoxy(screenw - 9, 4);
  myputs("\x12 cd ..  \x92");



  showdir(dir, screenh - 3, 2);

  while(1) {
    c = cgetc();

    switch(c) {

    case 'q':
    case A_BREAK:
      myputchar(147);
      ser_exit();
      return(EXIT_SUCCESS);
      break;

    case A_CR:
      de = getentry(dir, dir->current);
      if (de->size == 0) {
	if (de->type != DIRTYPE_DIR) {
	  (void) showrequester("Error", "This file is empty.", REQ_OK);
	} else {
	  if (changedir(de->name) == OK) {
	    freedir(dir);
	    if ((dir = getdir(direction, buffer, device)) == NULL) {
	      (void) showrequester("Error","Couldn't read directory.", REQ_OK);
	      ser_exit();
	      return(EXIT_FAILURE);
	    }
	    cleardir(screenh - 3, 2);
	    showdir(dir, screenh - 3, 2);
	  } else {
	    (void) showrequester("Error", "Couldn't change directory.", REQ_OK);
	  }
	}
      } else {
	type = imagetype(de->name, de->size);
	if (type == DOTHER) {
	  sprintf(msg, "%s is not a disk image.", de->name);
	  (void) showrequester("Error", msg, REQ_OK);
	} else {
	  sprintf(msg, "Write 15%d image to device %d?", type, device);
	  if (showrequester(de->name, msg, REQ_YESNO) == 1) {
	    if (writeimage(de->name, device, verify, buffer) == OK) {
	      (void) showrequester("Info", "Operation successful.", REQ_OK);
	    } else {
	      (void) showrequester("Error", "Operation failed!", REQ_OK);
	    }
	    resetscreen(titlestring);
	    showdir(dir, screenh - 3, 2);
	  }
	}
      }
      break;

    case A_CRSR_DOWN:
      if (dir->current < dir->numentries - 1) {
	printcurrent(dir, 2);
	++(dir->current);
	if (dir->current - dir->offset > screenh - 4) {
	  ++(dir->offset);
	  scrolldirup(screenh - 3, 2);
	}
	printrevcurrent(dir, 2);
      }
      break;

    case A_CRSR_UP:
      if (dir->current > 0) {
	printcurrent(dir, 2);
	--(dir->current);
	if (dir->current < dir->offset) {
	  dir->offset = dir->current;
	  scrolldirdown(screenh - 3, 2);
	}
	printrevcurrent(dir, 2);
      }
      break;

    case 'd':
      if (drv->next) {
	drv = drv->next;
      } else {
	while (drv->prev) {
	  drv = drv->prev;
	}
      }
      device = drv->dev;
      sprintf(msg, "\x18dev: %2d \x92", device);
      gotoxy(screenw - 9, 2);
      myputs(msg);
      break;

    case A_BACKSPACE:
      if (changedir("..") == OK) {
	freedir(dir);
	if ((dir = getdir(direction, buffer, device)) == NULL) {
	  (void) showrequester("Error","Couldn't read directory.", REQ_OK);
	  ser_exit();
	  return(EXIT_FAILURE);
	}
	cleardir(screenh - 3, 2);
	showdir(dir, screenh - 3, 2);
      } else {
	(void) showrequester("Error", "Couldn't change directory.", REQ_OK);
      }
      break;

    case 'S':
      ser_send(">@0:screenshot.prg\n\r", 20);
      if (ser_rx() == '!') {
	ser_tx(1000 & 255);
	ser_tx(1000 / 256);
	ser_tx(0x00);
	ser_tx(0x20);
	scr = (unsigned char *)1024;
	while (scr < (unsigned char *)2024) {
	  ser_tx(*scr++);
	}
      }
      break;

    }
    
  }
}
