#!/usr/bin/perl
#
# Serial Slave
# Copyright 2002 Per Olofsson, <MagerValp@cling.gu.se>
#
# This program is released under the Gnu General Public License
# See the file COPYING for the details
#
# history:
#  0.1    loading, directory listing, cd
#  0.2    wildcard support, minor code cleanup
#  0.3    code restructured, rudimentary d64 support
#  0.4    URL loading
#  0.5    saving, URL d64 mounting
#  0.6    . status code for positive result but no data
#         d64 file loading
#         code now uses strict parsing and is split into modules
#  0.7    fixed file corruption due to cr/lf conversion under win32
#         fixed broken d64 loading
#         added block mode reading
#         added serslave.cfg config file and commandline options
#  0.8    changed block protocol to allow multiple block transfers


# sir yes sir
use strict;

# nice warnings
use warnings qw(all);
no warnings qw(once);

# unbuffered output
$| = 1;

# URL loading support
use LWP::Simple;


# Version numbering scheme:
# * The version number is bumped by 0.1 whenever features are added
#   or the code changes are big enough.
# * An A suffix, for alpha, means that the version is buggy and
#   might be missing things that should be there.
# * A B suffix, for beta, means that it should be feature complete
#   but that it hasn't been tested extensively.
#   No suffix means that it has been tested and should be stable.

use constant VER => "0.8A";


# load code modules
require "modules/urls.pm";    # support for URL loading
require "modules/misc.pm";    # miscellaneous subroutines
require "modules/load.pm";    # handle LOAD requests
require "modules/save.pm";    #  handle SAVE requests
require "modules/dir.pm";     # directory handling
require "modules/d64.pm";     # d64 handling
require "modules/block.pm";   # block mode I/O
require "modules/jpg2mci.pm"; # jpg2mci conversion
require "modules/config.pm";  # config file parsing


# parse config file
readconfig("serslave.cfg");


# parse args
my $keyword;
my $value;
my $arg;
while ($arg = shift) {
  unless ($arg =~ /^--([a-z0-9]+)=?(.+)?$/) {
    print "Usage: [perl] serslave.perl [--keyword[=value]]\n";
    exit(0);
  }
  $keyword = lc($1);
  if(defined($2)) {
    $value = $2;
  } else {
    $value = 1;
  }

  if ($keyword eq "help") {
    print "Usage: [perl] serslave.perl [--keyword[=value]]\n";
    exit(0);
  } else {
    handlekwvl($keyword, $value, "command line", 1, 0);
  }
}


my $defaultport;

# try to guess the right OS
if (-d 'c:\\winnt' || -d 'c:\\windows' || -d 'd:\\winnt' || -d 'd:\\windows' || -d '\\winnt' || -d '\\windows') {
  print "Using Windows driver\n";
  require "modules/win32ser.pm";
  # win32 is lame
  use open IN => ":raw", OUT => ":raw";
  $::exesufx = ".exe";
  $::dirsep = "\\";
  $defaultport = "COM1";
} elsif (-d '/etc' && -d '/var' && -d '/usr' && -d '/lib') {
  print "Using UNIX driver\n";
  require "modules/unixser.pm";
  $::exesufx = "";
  $::dirsep = "/";
  $defaultport = "/dev/ttyS0";
} elsif (-d 'SYS:' && -d 'S:' && -d 'C:' && -d 'Devs:') {
  print "AmigaOS detected, please let me know if this works\n";
  require "modules/unixser.pm";
  $::exesufx = "";
  $::dirsep = "/";
  $defaultport = "/dev/ttyS0";
} else {
  print "Uknown operating system, using Device::SerialPort\n";
  print "Please let me know which OS you're running and if Serial Slave worked\n";
  require "modules/unixser.pm";
  $::exesufx = "";
  $::dirsep = "/";
  $defaultport = "/dev/ttyS0";
}


# defaults
$::baudrate = $::baudrate || 115200;
$::port = $::port || $defaultport;


# init serial connection
undef $::ob;
my $baud = setupserial($::baudrate, $::port);
if ($baud == 0) {
  print "Failed to open serial port at $::baudrate baud\n";
  exit 1;
}
print "serslave v" . VER . " at $baud baud\n";


# Globals

$::cd = "";              # set to a filename if we've CD'd into something magical
@::block = ();           # block buffer
$::blockmode = 0;        # 0 closed, 1 read, 2 write
$::debug = 1;            # enable debug o]* Use of uninitialized value in substutput
$::exesufx = $::exesufx; # .exe suffix?
$::dirsep = $::dirsep;   # / or \ dir separator
$::jpg2mci = 0;          # flag to keep track of jpg conversion



# wait for requests
my ($count, $result);
my $oldresult = "";
print "Press Ctrl-C to quit\nREADY.\n";
while (1) { # check for q instead...?
  fsleep(0.01);

  ($count, $result) = $::ob->read(100);
  next if ($count == 0);
  print pretty($result);

  $result = $oldresult . $result;
  $oldresult = "";

  if ($result =~ s/^(.*?)\r\n//) {
    $oldresult = $result;
    $result = $1;

    if ($result =~ /^\<(.+)/s) {       # < for load
      loadreq($1);
    } elsif ($result =~ /^\>(.+)/s) {  # > for save
      savereq($1);
    } elsif ($result =~ /^BL(.+)/s) {  # BL for block mode load
      blockload($1);
    } elsif ($result =~ /^BS(.+)/s) {  # BS for block mode save
      blocksave($1);
    } elsif ($result =~ /^BR(...)/s) { # BR for block read
      blockread($1);
    } elsif ($result =~ /^BW(..)/s) {  # BW for block write
      blockwrite($1);
    } elsif ($result =~ /^BC/s) {      # BC for block close
      blockclose($1);
    } elsif ($result =~ /^\?/s) {      # ? for ping
      print "ping!\n";
      send_wsize(atopet("Serial Slave v" . VER));
    } else {
      print "unknown request: " . pretty($result) . "\n";
    }

  } else {
    $oldresult = $result;
  }

  if ($oldresult eq "") {
    print "READY.\n";
  }
}
exit(0);
